/*
 * Copyright (c) 2015, Mellanox Technologies. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include "en.h"

static int mlx5e_test_loopback(struct mlx5_core_dev *mdev)
{
	/* not supported yet */
	return 0;
}

static int mlx5e_test_link_state(struct mlx5_core_dev *mdev)
{
	u8 port_state;

	port_state = mlx5_query_vport_state(mdev,
		MLX5_QUERY_VPORT_STATE_IN_OP_MOD_VNIC_VPORT);

	if (port_state == VPORT_STATE_UP)
		return 0;
	return 1;
}

static int mlx5e_test_link_speed(struct mlx5_core_dev *mdev)
{
	u32 out[MLX5_ST_SZ_DW(ptys_reg)];
	u32 eth_proto_oper;
	int err;
	int i;

	err = mlx5_query_port_ptys(mdev, out, sizeof(out), MLX5_PTYS_EN);
	if (err)
		return 1;

	eth_proto_oper  = MLX5_GET(ptys_reg, out, eth_proto_oper);

	for (i = 0; i < MLX5E_LINK_MODES_NUMBER; ++i) {
		if (eth_proto_oper & MLX5E_PROT_MASK(i))
			return 0;
	}
	return 1;
}

static int mlx5e_test_health_info(struct mlx5_core_dev *mdev)
{
	struct mlx5_core_health *health = &mdev->priv.health;

	if (health->sick)
		return 1;
	return 0;
}

void mlx5e_self_test(struct net_device *dev,
		     struct ethtool_test *etest,
		     u64 *buf)
{
	struct mlx5e_priv *priv = netdev_priv(dev);
	struct mlx5_core_dev *mdev = priv->mdev;

	memset(buf, 0, sizeof(u64) * MLX5E_NUM_SELF_TEST);

	if (netif_carrier_ok(dev)) {
		buf[MLX5E_LINK_SPEED]	= mlx5e_test_link_speed(mdev);
		buf[MLX5E_LINK_STATE]	= mlx5e_test_link_state(mdev);
	}

	buf[MLX5E_HEALTH_INFO]	= mlx5e_test_health_info(mdev);
	buf[MLX5E_LOOPBACK]	= mlx5e_test_loopback(mdev);
}
